/*
 * Copyright (C) 2014 by
 *   MetraLabs GmbH (MLAB), GERMANY
 * and
 *   Neuroinformatics and Cognitive Robotics Labs (NICR) at TU Ilmenau, GERMANY
 * All rights reserved.
 *
 * Contact: info@mira-project.org
 *
 * Commercial Usage:
 *   Licensees holding valid commercial licenses may use this file in
 *   accordance with the commercial license agreement provided with the
 *   software or, alternatively, in accordance with the terms contained in
 *   a written agreement between you and MLAB or NICR.
 *
 * GNU General Public License Usage:
 *   Alternatively, this file may be used under the terms of the GNU
 *   General Public License version 3.0 as published by the Free Software
 *   Foundation and appearing in the file LICENSE.GPL3 included in the
 *   packaging of this file. Please review the following information to
 *   ensure the GNU General Public License version 3.0 requirements will be
 *   met: http://www.gnu.org/copyleft/gpl.html.
 *   Alternatively you may (at your option) use any later version of the GNU
 *   General Public License if such license has been publicly approved by
 *   MLAB and NICR (or its successors, if any).
 *
 * IN NO EVENT SHALL "MLAB" OR "NICR" BE LIABLE TO ANY PARTY FOR DIRECT,
 * INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES ARISING OUT OF
 * THE USE OF THIS SOFTWARE AND ITS DOCUMENTATION, EVEN IF "MLAB" OR
 * "NICR" HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * "MLAB" AND "NICR" SPECIFICALLY DISCLAIM ANY WARRANTIES, INCLUDING,
 * BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 * FITNESS FOR A PARTICULAR PURPOSE. THE SOFTWARE PROVIDED HEREUNDER IS
 * ON AN "AS IS" BASIS, AND "MLAB" AND "NICR" HAVE NO OBLIGATION TO
 * PROVIDE MAINTENANCE, SUPPORT, UPDATES, ENHANCEMENTS OR MODIFICATIONS.
 */

/**
 * @file PoseLineEdit.h
 *    A widget that allows compact editing of a Pose2.
 *
 * @author Christian Reuther
 * @date   2014/09/09
 */

#ifndef _MIRA_POSELINEEDIT_H_
#define _MIRA_POSELINEEDIT_H_

#include <QWidget>
#include <QDoubleSpinBox>

#include <widgets/GuiWidgetsExports.h>
#include <transform/Pose.h>

namespace mira {

///////////////////////////////////////////////////////////////////////////////

class MIRA_GUI_WIDGETS_EXPORT Pose2LineEdit : public QWidget
{
	Q_OBJECT
public:
	Pose2LineEdit(QWidget *parent = NULL);

	Pose2 value() const;
	bool isReadOnly() const;

	void setValue(const Pose2& pose);
	void setLimits(double minX, double minY, double minPhi, double maxX, double maxY, double maxPhi);
	void setPrecision(double stepX, double stepY, double stepPhi, int decimalsX, int decimalsY, int decimalsPhi);
	void setReadOnly(bool ro);

protected:
	void createGUI();
	void validateWidgets() const;

	QDoubleSpinBox* mEditX;
	QDoubleSpinBox* mEditY;
	QDoubleSpinBox* mEditPhi;
};

///////////////////////////////////////////////////////////////////////////////

}

#endif
