/*
 * Copyright (C) 2012 by
 *   MetraLabs GmbH (MLAB), GERMANY
 * and
 *   Neuroinformatics and Cognitive Robotics Labs (NICR) at TU Ilmenau, GERMANY
 * All rights reserved.
 *
 * Contact: info@mira-project.org
 *
 * Commercial Usage:
 *   Licensees holding valid commercial licenses may use this file in
 *   accordance with the commercial license agreement provided with the
 *   software or, alternatively, in accordance with the terms contained in
 *   a written agreement between you and MLAB or NICR.
 *
 * GNU General Public License Usage:
 *   Alternatively, this file may be used under the terms of the GNU
 *   General Public License version 3.0 as published by the Free Software
 *   Foundation and appearing in the file LICENSE.GPL3 included in the
 *   packaging of this file. Please review the following information to
 *   ensure the GNU General Public License version 3.0 requirements will be
 *   met: http://www.gnu.org/copyleft/gpl.html.
 *   Alternatively you may (at your option) use any later version of the GNU
 *   General Public License if such license has been publicly approved by
 *   MLAB and NICR (or its successors, if any).
 *
 * IN NO EVENT SHALL "MLAB" OR "NICR" BE LIABLE TO ANY PARTY FOR DIRECT,
 * INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES ARISING OUT OF
 * THE USE OF THIS SOFTWARE AND ITS DOCUMENTATION, EVEN IF "MLAB" OR
 * "NICR" HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * "MLAB" AND "NICR" SPECIFICALLY DISCLAIM ANY WARRANTIES, INCLUDING,
 * BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 * FITNESS FOR A PARTICULAR PURPOSE. THE SOFTWARE PROVIDED HEREUNDER IS
 * ON AN "AS IS" BASIS, AND "MLAB" AND "NICR" HAVE NO OBLIGATION TO
 * PROVIDE MAINTENANCE, SUPPORT, UPDATES, ENHANCEMENTS OR MODIFICATIONS.
 */

/**
 * @file Preferences.h
 *    .
 *
 * @author Erik Einhorn
 * @date   12/02/2011
 */

#ifndef _MIRA_PREFERENCES_H_
#define _MIRA_PREFERENCES_H_

#ifndef Q_MOC_RUN
#include <boost/assign/list_of.hpp>
#endif

#include <serialization/adapters/boost/array.hpp>

#include <serialization/DefaultInitializer.h>

namespace mira {

///////////////////////////////////////////////////////////////////////////////

class Preferences
{
public:

	Preferences() {
		MIRA_INITIALIZE_THIS;
	}

	template<typename Reflector>
	void reflect(Reflector& r)
	{
		r.member("AutosaveWorkspace",autosaveWorkspace,"",true);
		r.member("AutosaveWorkspaceInterval",autosaveWorkspaceInterval,"",10);
		r.member("HideEditorWindowDecorations",hideEditorWindowDecorations,"",false);
		r.member("Corners",corners,"",boost::assign::list_of(true)(true)(true)(true));
		r.member("LockWorkspace",lockWorkspace,"",false);
	}


public:

	bool autosaveWorkspace;
	int  autosaveWorkspaceInterval;

	bool hideEditorWindowDecorations;

	bool lockWorkspace;

	enum Corners {
		TopLeft=0,
		TopRight,
		BottomLeft,
		BottomRight,
		CornerCount
	};

	// contains true, if corner belongs to the left or right side; false, if it belongs to top or bottom
	typedef boost::array<bool,CornerCount> CornerSettings;
	CornerSettings corners;


};

///////////////////////////////////////////////////////////////////////////////

} // namespace

#endif
